﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Infragistics.Documents;
using Infragistics.SyntaxParsing;

namespace ConsoleApplication1
{
	class Program
	{
		static void Main(string[] args)
		{
			var grammar = new Grammar();
			grammar.FromEbnf(
@"
?
<Grammar>
	<Name>MyGrammar</Name>
	<StartSymbol>Document</StartSymbol>
</Grammar>
?;

?
<TerminalSymbols>
	<TerminalSymbol> <Name>CommaToken</Name> <Value>,</Value> </TerminalSymbol>
	<TerminalSymbol> <Name>DotToken</Name> <Value>.</Value> </TerminalSymbol>
	<TerminalSymbol> <Name>ExclamationPointToken</Name> <Value>!</Value> </TerminalSymbol>

	<!-- The question mark must be escaped below so the special sequence is not ended. -->
	<TerminalSymbol> <Name>QuestionMarkToken</Name> <Value>&#x3F;</Value> </TerminalSymbol>

	<TerminalSymbol> <Name>SemicolonToken</Name> <Value>;</Value> </TerminalSymbol>
	<TerminalSymbol> <Name>WordToken</Name> <Value>\w+</Value> <Comparison>RegularExpression</Comparison> </TerminalSymbol>
</TerminalSymbols>
?;

Document = 
	{Sentence};
Sentence = 
	Phrase, {PhraseSeparatingPunctuation, Phrase}, SentenceEndingPunctuation;
SentenceEndingPunctuation =
	DotToken
	| ExclamationPointToken
	| QuestionMarkToken;
PhraseSeparatingPunctuation =
	CommaToken
	| SemicolonToken;
Phrase =
	_words;
_words =
	WordToken |
	WordToken, _words;
");

			CustomLanguage language = new CustomLanguage(grammar);

			TextDocument document = new TextDocument();
			document.Language = language;

			document.Append(
@"
Hello World! This sentence has multiple phrases; they are separated by a semicolon.
");

			// Force a synchronous parse of the document.
			document.Parse();

			SyntaxTree tree = document.SyntaxTree;
			DebugWriteTree(tree.RootNode);
		}

		private static void DebugWriteTree(SyntaxNode node)
		{
			if (node.IsTerminalNode)
				Debug.WriteLine(node.Symbol.Name + ": " + node.GetText());
			else
				Debug.WriteLine(node.Symbol.Name);

			Debug.Indent();

			for (int i = 0; i < node.ChildNodeCount; i++)
				DebugWriteTree(node.GetChildNodeAtIndex(i));

			Debug.Unindent();
		}
	}
}
